package util;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import cn.gbase8.gswt.GBaseConnectionParamater;
import cn.gbase8.gswt.QueryResult;

public class Common {
	public static boolean isBlank(String str) {
		return str == null || str.trim().length() == 0;
	}

	public static String noNull(String str) {
		return str == null ? "" : str;
	}

	public static String noNull(Object obj) {
		return obj == null ? "" : noNull(obj.toString());
	}

	public static String noNull(String str, String defaultValue) {
		return str == null ? defaultValue : str;
	}

	public static String noNull(Object obj, String defaultValue) {
		return obj == null ? defaultValue : noNull(obj.toString());
	}

	public static QueryResult resultSetToMap(ResultSet rs) throws Exception {
		return resultSetToMap(rs, 1000);
	}

	public static QueryResult resultSetToMap(ResultSet rs, long limit) throws Exception {
		if (rs == null) {
			return null;
		}
		QueryResult qr = new QueryResult();
		List<List<String>> rtn = new LinkedList<List<String>>();
		qr.result = rtn;

		ResultSetMetaData rsm = rs.getMetaData();
		int columns = rsm.getColumnCount();
		List<String> list = new ArrayList<String>(columns);
		for (int i = 1; i <= columns; i++) {
			list.add(rsm.getColumnLabel(i));
		}
		rtn.add(list);
		int rows = 0;
		while (rs.next()) {
			if (rows >= limit) {
				break;
			}
			list = new ArrayList<String>(columns);
			for (int i = 1; i <= columns; i++) {
				list.add(rs.getString(i));
			}
			rtn.add(list);
			rows++;
		}
		return qr;
	}

	public static List<QueryResult> sqlToMap(GBaseConnectionParamater gcp, String sql) {
		try {
			return sqlToMap(gcp, sql, 1000, 60);
		} catch (Exception ex) {
			ex.printStackTrace();
			return null;
		}
	}

	public static List<QueryResult> sqlToMap(GBaseConnectionParamater gcp, String sql, long limit, int timeout) {
		Connection con = null;
		Statement stat = null;
		ResultSet rs = null;
		List<QueryResult> qrs = new LinkedList<QueryResult>();
		long beginTime = System.currentTimeMillis();
		try {
			con = Common.getConnection(gcp);

			// 避免大结果集导致内存不足，采用流式处理
			stat = con.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
			stat.setFetchSize(Integer.MIN_VALUE);
			stat.setQueryTimeout(timeout);

			stat.execute(sql);
			do {
				rs = stat.getResultSet();

				if (rs == null) {
					QueryResult qr = new QueryResult();
					List<List<String>> rtn = new LinkedList<List<String>>();

					List<String> list = new ArrayList<String>(1);
					list.add("执行结果");
					rtn.add(list);
					list = new ArrayList<String>(1);
					list.add(stat.getUpdateCount() == -1 ? "失败" : "成功");
					rtn.add(list);

					qr.result = rtn;
					qrs.add(qr);
				} else {
					qrs.add(Common.resultSetToMap(rs, limit));
				}
			} while (stat.getMoreResults() || stat.getUpdateCount() != -1);

			return qrs;
		} catch (Exception ex) {
			ex.printStackTrace();
			// 处理SQL报错
			QueryResult qr = new QueryResult();
			qrs.add(qr);
			qr.message = ex.getLocalizedMessage();
			return qrs;
		} finally {
			closeConnection(con, null, null);
			// 执行耗时
			long queryTime = System.currentTimeMillis() - beginTime;
			QueryResult qr = new QueryResult();
			List<List<String>> rtn = new LinkedList<List<String>>();
			List<String> list = new ArrayList<String>(1);
			list.add("总耗时:" + (new BigDecimal(queryTime).divide(new BigDecimal(1000))) + "秒");
			rtn.add(list);
			qr.result = rtn;
			qrs.add(qr);
		}
	}

	public static void closeConnection(Connection con, Statement stat, ResultSet rs) {
		if (rs != null) {
			try {
				rs.close();
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
		if (stat != null) {
			try {
				stat.close();
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
		if (con != null) {
			try {
				con.close();
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
	}

	private static String getURL(GBaseConnectionParamater gcp) {
		return "jdbc:gbase://" + gcp.serverIP + ":5258/" + gcp.dbName + "?user=" + gcp.username + "&password="
				+ gcp.password + "&useOldAliasMetadataBehavior=false&allowMultiQueries=true";

	}

	public static boolean checkConnection(GBaseConnectionParamater gcp) {
		Connection con = null;
		try {
			con = getConnection(gcp);
			return con != null;
		} catch (Exception ex) {
			ex.printStackTrace();
			return false;
		} finally {
			if (con != null) {
				try {
					con.close();
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		}
	}

	public static Connection getConnection(GBaseConnectionParamater gcp) throws Exception {
		String URL = getURL(gcp);
		Class.forName("com.gbase.jdbc.Driver");

		return DriverManager.getConnection(URL);
	}

	public static String md5utf8(String str) {
		try {
			MessageDigest md = MessageDigest.getInstance("MD5");
			md.update(str.getBytes("UTF8"));// 计算md5函数
			/**
			 * digest()最后确定返回md5 hash值，返回值为8位字符串。 因为md5 hash值是16位的hex值，实际上就是8位的字符
			 * BigInteger函数则将8位的字符串转换成16位hex值，用字符串来表示；得到字符串形式的hash值
			 * 一个byte是八位二进制，也就是2位十六进制字符（2的8次方等于16的2次方）
			 */
			return new BigInteger(1, md.digest()).toString(16);// 16是表示转换为16进制数

		} catch (Exception ex) {
			ex.printStackTrace();
			return null;
		}
	}
}
